


/*Cuong 's function prototypes*/

/*

Function prototypes to implementing the extension version of Graphplan to handle Resources.
- Creating graph
- finding fact, fluent and action mutxes
- extracting a plan from graphplan

are designed and programed by Cuong Pham (cuong@nmsu.edu)
Under the supervision of Dr. Son Tran (tson@cs.nmsu.edu)

*/




#ifndef _GRAPH_H
#define _GRAPH_H






void init();

int tryAction (int key, int time); /*try if the action is executable at time*/

int checkFact(int i, Fact *f);

int factOK(int i,int time); /*check if a->preconds[i] is true at time t*/

int checkFluent();

void insertAction(int time);/*insert all  action <>no-op into ActionTable[time]*/
float precondExpNode_Eval(ExpNode *n,int time);

Bool checkPreActionKey(FluentValues *f, int key);

Bool checkEffActionKey(FluentValues *f, int key);

void makeEffect(int k, int time);

void copyFact(int time);

void copyFluent(int time);

void copyAction(int time);

float getFluentValueEff(Fluent *f,int time, int key);

 /*to compute effect metric*/

float getFluentValuePre(Fluent *f,int time, int key); 
/*to compute pre metric*/


Bool factEqual(Fact *f1, Fact *f2);
 /*To eliminate redundant facts and find mutexes*/

Bool foundFact(Fact *f, int time, int key);

Bool foundAction(int k, int time);

void CreateGraphLevel(int time);

int createGraph();

void ExecutableAction(int time);

Bool FluentValueEqual(FluentValues *f1, FluentValues *f2);

Bool foundFluentValue(FluentValues *f, int time, int key); /*to prune*/


ActionVertex *getAction_in_Table( int key, int time);

Action *getAction_in_List( int key);

ActionVertex *new_ActionVertex( void );

/* To reindex facts at time t eg. At(x,y).index=3  ==> not at(x,y).index=-3*/
/*reindex is actually to find a mutex between two facts at time t*/

Bool NearlyEqualFact(Fact *f1, Fact *f2);

/*functions to evaluate numeric preconds */


void ReIndexFact(int time);

int getFluentValueIndex(ExpNode *n, int time);

/*input: add[i] or dels[i], it returns a fact index*/
int getFactIndex(int k, Bool neg,int time);

/*update outfact list for actionvertex key*/
void UpdateOutFact_ActionVertex(ActionVertex *g,int time);

/*update for all actionvertexes at time */
void UpdateOutFact(int time);


/* To remove the track of fluenttable[time] for new action */
void RefreshFluent(int time);

Bool Numeric_Fluent_Eval (int time, int preActionKey, ExpNode *n1, Comparator comp, ExpNode *n2);
Bool foundFluentAtTime(Fluent *f, FluentValues *g);

FluentValues * getFluentValueAtIndex(Fluent *f, int index, int time);


Bool checkNumericPreconds (float u, Comparator comp, float v);

float ExpNode_Eval(ExpNode *n);

float NumericEffect_Eval(int time, int n1, NumericEffectType op, ExpNode *n2);

Bool isFluent (ExpNode *n);


/*for finding mutex*/
Facts *getFactAtTime(int index, int time);

Bool TwoFactMutex(Facts *f1, Facts *f2, int time);

/*if two actions are mutex, both of them should be updated*/
void UpdateTwoActionMutex(ActionVertex *a, ActionVertex *b, int time);

void updateActionmutex(ActionVertex *a, int time);

Bool AreMutexFact(Facts *f1, Facts *f2);

/*two facts were marked as mutex as the previous level*/
Bool TwoFactMarkedMutexAtTime(Facts *f1, Facts *f2, int time);

/*check if the mutex action has been updated */
Bool AlreadyUpdateActionMutex(ActionVertex * a, int key);


void findMutexFactAtTime(int time); 

/*update all actions mutexed within a time*/
void updateMutexActionAtTime(int time);

Bool checkTwoActionMutex(ActionVertex *a, ActionVertex *b, int time);

/*just need two actions as input....*/

void findMutexActionAtTime(int time);

/*for fact mutex now */
Bool AlreadyUpdateFactMutex(Fact * f, int index);

void UpdateTwoFactMutex(Fact *f1, Fact *f2, int time);

void updateMutexFactAtTime(int time);


/*for fluent mutex now*/

Bool NearlyEqualFluent(FluentValues *f1, FluentValues *f2);

Bool TwoFluentMutex(FluentValues *f1, FluentValues *f2, int time);

void UpdateTwoFluentMutex(FluentValues *f1, FluentValues *f2, int time);

void updateMutexFluentAtTime(int time);

FluentValues *getFluentAtTime(int index, int time);



/* functions for extracting */


int getGoalFactIndex(int k, Bool neg,int time);

ActionPlan *new_ActionPlan( void );

int reachGoal(int time);

Bool stopExtendingGraph(int time);

Bool AreMutexAction(int n1, int n2, int time);

Bool CanInsertAction_toPlan(int key, int time);

void createBasicPlan(int time);

void createFactGoal(int time);

Bool checkExistFactGoal(int t, int time);


void createFluentGoal(int time);

Bool checkExistFluentGoal(int index, int time);


void ExtractSolution();

void pruningActionSet(int time);

void removeAction(ActionPlan *a,int time);

/*print */

void print_Graph(int time);


/*build plan*/

GoalType *new_Goal( void );

void extractInit();

void createfinalGoal(int time);

void markfactGoal(ActionVertex *a,int time);

void unmarkfactGoal(ActionVertex *a,int time);

void markfinalGoal(int numGoal, int time);

Bool twoActionMutex(ActionVertex *a,ActionVertex *b);

Bool mutexActionPlan(ActionVertex *a, int time); 

Bool compatible(ActionVertex *a, int time);

/*check if can insert action to plan*/

Bool checkOk();
Bool checkOk1();

void addAction_toplan(ActionVertex *p,GoalType *g, int time);

void removeActionplan(ActionVertex *p,int time);

ActionVertex * pickupAction(GoalType *g, int time);

int makeGoal(ActionVertex *a, int time);

void print_Plan();

int Try(GoalType *g , int n,int i);

int Try1(GoalType *g , int n,int i);

void countGraph();

/**End of function prototypes*/

/***********End of Declaration*******************/

#endif


/*
A valid plan testing:


0:LOAD ALEX R2 LONDON
1:LOAD JASON R2 LONDON

2:UNLOAD ALEX R2 LONDON
3:LOAD ALEX R1 LONDON
4:MOVE R2 LONDON JFK
5:MOVE R1 LONDON PARIS
6:UNLOAD ALEX R1 PARIS
7:UNLOAD JASON R2 JFK


*/
